@@ -27,8 +27,6 @@ Gerhard Schroeder       0.76      0.76      0.76        25
 ================== ============ ======= ========== =======
 
 """
-from __future__ import print_function
-
 from time import time
 import logging
 import matplotlib.pyplot as plt
@@ -109,7 +107,7 @@ t0 = time()
 param_grid = {'C': [1e3, 5e3, 1e4, 5e4, 1e5],
               'gamma': [0.0001, 0.0005, 0.001, 0.005, 0.01, 0.1], }
 clf = GridSearchCV(SVC(kernel='rbf', class_weight='balanced'),
-                   param_grid, cv=5)
+                   param_grid, cv=5, iid=False)
 clf = clf.fit(X_train_pca, y_train)
 print("done in %0.3fs" % (time() - t0))
 print("Best estimator found by grid search:")
