@@ -13,7 +13,7 @@ density estimation problem and use the `OneClassSVM` provided
 by the package `sklearn.svm` as our modeling tool.
 The dataset is provided by Phillips et. al. (2006).
 If available, the example uses
-`basemap <http://matplotlib.org/basemap>`_
+`basemap <https://matplotlib.org/basemap/>`_
 to plot the coast lines and national boundaries of South America.
 
 The two species are:
@@ -41,8 +41,6 @@ References
 #
 # License: BSD 3 clause
 
-from __future__ import print_function
-
 from time import time
 
 import numpy as np
