@@ -59,13 +59,11 @@ is to position the labels minimizing overlap. For this we use an
 heuristic based on the direction of the nearest neighbor along each
 axis.
 """
-from __future__ import print_function
 
 # Author: Gael Varoquaux gael.varoquaux@normalesup.org
 # License: BSD 3 clause
 
 import sys
-from datetime import datetime
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -85,8 +83,6 @@ print(__doc__)
 # that we get high-tech firms, and before the 2008 crash). This kind of
 # historical data can be obtained for from APIs like the quandl.com and
 # alphavantage.co ones.
-start_date = datetime(2003, 1, 1).date()
-end_date = datetime(2008, 1, 1).date()
 
 symbol_dict = {
     'TOT': 'Total',
