@@ -18,7 +18,8 @@ from sklearn.datasets.samples_generator import make_blobs
 X, Y = make_blobs(n_samples=50, centers=2, random_state=0, cluster_std=0.60)
 
 # fit the model
-clf = SGDClassifier(loss="hinge", alpha=0.01, max_iter=200, fit_intercept=True)
+clf = SGDClassifier(loss="hinge", alpha=0.01, max_iter=200,
+                    fit_intercept=True, tol=1e-3)
 clf.fit(X, Y)
 
 # plot the line, the points, and the nearest vectors to the plane
