@@ -43,11 +43,11 @@ scoring = {'AUC': 'roc_auc', 'Accuracy': make_scorer(accuracy_score)}
 # Setting refit='AUC', refits an estimator on the whole dataset with the
 # parameter setting that has the best cross-validated AUC score.
 # That estimator is made available at ``gs.best_estimator_`` along with
-# parameters like ``gs.best_score_``, ``gs.best_parameters_`` and
+# parameters like ``gs.best_score_``, ``gs.best_params_`` and
 # ``gs.best_index_``
 gs = GridSearchCV(DecisionTreeClassifier(random_state=42),
                   param_grid={'min_samples_split': range(2, 403, 10)},
-                  scoring=scoring, cv=5, refit='AUC')
+                  scoring=scoring, cv=5, refit='AUC', return_train_score=True)
 gs.fit(X, y)
 results = gs.cv_results_
 
@@ -61,9 +61,8 @@ plt.title("GridSearchCV evaluating using multiple scorers simultaneously",
 
 plt.xlabel("min_samples_split")
 plt.ylabel("Score")
-plt.grid()
 
-ax = plt.axes()
+ax = plt.gca()
 ax.set_xlim(0, 402)
 ax.set_ylim(0.73, 1)
 
@@ -93,5 +92,5 @@ for scorer, color in zip(sorted(scoring), ['g', 'k']):
                 (X_axis[best_index], best_score + 0.005))
 
 plt.legend(loc="best")
-plt.grid('off')
+plt.grid(False)
 plt.show()
