@@ -32,9 +32,7 @@ from sklearn.neural_network import MLPClassifier
 h = .02  # step size in the mesh
 
 alphas = np.logspace(-5, 3, 5)
-names = []
-for i in alphas:
-    names.append('alpha ' + str(i))
+names = ['alpha ' + str(i) for i in alphas]
 
 classifiers = []
 for i in alphas:
