@@ -8,7 +8,6 @@ https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.da
 
 """
 
-import sys
 import errno
 from gzip import GzipFile
 import logging
@@ -22,12 +21,12 @@ from .base import _fetch_remote
 from .base import get_data_home
 from .base import RemoteFileMetadata
 from ..utils import Bunch
-from ..externals import joblib, six
+from ..utils import _joblib
 from ..utils import check_random_state
 from ..utils import shuffle as shuffle_method
 
 # The original data can be found at:
-# http://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data.gz
+# https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data.gz
 ARCHIVE = RemoteFileMetadata(
     filename='kddcup99_data',
     url='https://ndownloader.figshare.com/files/5976045',
@@ -35,7 +34,7 @@ ARCHIVE = RemoteFileMetadata(
               '343652c9db428893e7494f837b274292'))
 
 # The original data can be found at:
-# http://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data_10_percent.gz
+# https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data_10_percent.gz
 ARCHIVE_10_PERCENT = RemoteFileMetadata(
     filename='kddcup99_10_data',
     url='https://ndownloader.figshare.com/files/5976042',
@@ -140,9 +139,9 @@ def fetch_kddcup99(subset=None, data_home=None, shuffle=False,
         data = np.c_[data[s, :11], data[s, 12:]]
         target = target[s]
 
-        data[:, 0] = np.log((data[:, 0] + 0.1).astype(float))
-        data[:, 4] = np.log((data[:, 4] + 0.1).astype(float))
-        data[:, 5] = np.log((data[:, 5] + 0.1).astype(float))
+        data[:, 0] = np.log((data[:, 0] + 0.1).astype(float, copy=False))
+        data[:, 4] = np.log((data[:, 4] + 0.1).astype(float, copy=False))
+        data[:, 5] = np.log((data[:, 5] + 0.1).astype(float, copy=False))
 
         if subset == 'http':
             s = data[:, 2] == b'http'
@@ -204,14 +203,7 @@ def _fetch_brute_kddcup99(data_home=None,
     """
 
     data_home = get_data_home(data_home=data_home)
-    if sys.version_info[0] == 3:
-        # The zlib compression format use by joblib is not compatible when
-        # switching from Python 2 to Python 3, let us use a separate folder
-        # under Python 3:
-        dir_suffix = "-py3"
-    else:
-        # Backward compat for Python 2 users
-        dir_suffix = ""
+    dir_suffix = "-py3"
 
     if percent10:
         kddcup_dir = join(data_home, "kddcup99_10" + dir_suffix)
@@ -276,8 +268,7 @@ def _fetch_brute_kddcup99(data_home=None,
         file_ = GzipFile(filename=archive_path, mode='r')
         Xy = []
         for line in file_.readlines():
-            if six.PY3:
-                line = line.decode()
+            line = line.decode()
             Xy.append(line.replace('\n', '').split(','))
         file_.close()
         logger.debug('extraction done')
@@ -293,8 +284,8 @@ def _fetch_brute_kddcup99(data_home=None,
         # (error: 'Incorrect data length while decompressing[...] the file
         #  could be corrupted.')
 
-        joblib.dump(X, samples_path, compress=0)
-        joblib.dump(y, targets_path, compress=0)
+        _joblib.dump(X, samples_path, compress=0)
+        _joblib.dump(y, targets_path, compress=0)
     elif not available:
         if not download_if_missing:
             raise IOError("Data not found and `download_if_missing` is False")
@@ -302,8 +293,8 @@ def _fetch_brute_kddcup99(data_home=None,
     try:
         X, y
     except NameError:
-        X = joblib.load(samples_path)
-        y = joblib.load(targets_path)
+        X = _joblib.load(samples_path)
+        y = _joblib.load(targets_path)
 
     return Bunch(data=X, target=y)
 
