@@ -10,7 +10,7 @@ from sklearn.datasets import make_classification, load_digits, make_blobs
 from sklearn.svm.tests import test_svm
 from sklearn.exceptions import ConvergenceWarning
 from sklearn.utils.extmath import safe_sparse_dot
-from sklearn.utils.testing import (assert_raises, assert_false, assert_warns,
+from sklearn.utils.testing import (assert_raises, assert_warns,
                                    assert_raise_message, ignore_warnings,
                                    skip_if_32bit)
 
@@ -113,21 +113,20 @@ def test_unsorted_indices():
 
     # reverse each row's indices
     def scramble_indices(X):
-        new_data = np.empty_like(X.data)
-        new_indices = np.empty_like(X.indices)
+        new_data = []
+        new_indices = []
         for i in range(1, len(X.indptr)):
             row_slice = slice(*X.indptr[i - 1: i + 1])
-            new_data[row_slice] = X.data[row_slice][::-1]
-            new_indices[row_slice] = X.indices[row_slice][::-1]
+            new_data.extend(X.data[row_slice][::-1])
+            new_indices.extend(X.indices[row_slice][::-1])
         return sparse.csr_matrix((new_data, new_indices, X.indptr),
                                  shape=X.shape)
 
     X_sparse_unsorted = scramble_indices(X_sparse)
     X_test_unsorted = scramble_indices(X_test)
 
-    # make sure we scramble the indices
-    assert_false(X_sparse_unsorted.has_sorted_indices)
-    assert_false(X_test_unsorted.has_sorted_indices)
+    assert not X_sparse_unsorted.has_sorted_indices
+    assert not X_test_unsorted.has_sorted_indices
 
     unsorted_svc = svm.SVC(kernel='linear', probability=True,
                            random_state=0).fit(X_sparse_unsorted, y)
